"""
手动获取奥兰系统验证码的数据集
"""
import hashlib
import time
from io import BytesIO

import requests
from PIL import Image

from njupt.urls import URL
from njupt.utils.aolan.aolan_captcha import letters

for i in range(100):
    r = requests.get(URL.aolan_captcha())
    im = Image.open(BytesIO(r.content))
    # 灰度处理
    imgry = im.convert('L')
    # 二值处理
    threshold = 220
    table = []
    for i in range(256):
        table.append(0 if i < threshold else 1)
    out = imgry.point(table, '1')
    out.show()
    numbers = input("输入数值")
    if numbers == "-1":
        continue
    count = 0
    for letter, value in zip(letters, numbers):
        m = hashlib.md5()
        m.update("{}{}".format(time.time(), count).encode('utf8'))
        temp = out.crop((letter[0], 0, letter[1], out.size[1]))
        temp.save('captcha_chars/{}/{}.png'.format(value, m.hexdigest()))
        count += 1



# encoding: utf-8
import re

from bs4 import BeautifulSoup

from njupt.decorators.zhengfang_logined import zhengfang_logined
from njupt.models.base import Model
from njupt.urls import URL
from njupt.utils import ZhengfangCaptcha

week_re = re.compile(r'第(\d+)-(\d+)周')
courser_indexs_re = re.compile(r'第(\d+(?:,\d+)*)节')
chinese_rome = {
    '一': 1,
    '二': 2,
    '三': 3,
    '四': 4,
    '五': 5,
    '六': 6,
    '七': 7
}


class Zhengfang(Model):
    def __init__(self, account=None, password=None):
        super(Zhengfang, self).__init__()
        if account and password:
            self.account = account
            self.login(account, password)

    @zhengfang_logined
    def get_grade(self):
        """
        获取等级考试成绩信息
        :return: list
                [
                    {'date': '20151219',  # 考试日期
                     'name': '全国大学英语四级考试',  # 考试名称
                     'number': '320082152113313',  # 准考证号
                     'score': '547',  # 成绩
                     'semester': '1',  # 学期
                     'year': '2015-2016'  # 学年
                     },
                ]
        """
        soup = self._url2soup(method='get', url=URL.zhengfang_grade(self.account))
        results = []
        for tr in soup.select("#DataGrid1 > tr")[1:]:
            names = ['year', 'semester', 'name', 'number', 'date', 'score']
            result = {}
            for name, td in zip(names, tr.select('td')[:6]):
                result[name] = td.text
            results.append(result)
        return results

    @zhengfang_logined
    def get_schedule(self, week, year=None, semester=None):
        """
        获取指定学期指定周的课表（不传入年份和学期则默认当前学期）
        :param year: 学年 格式为 "2017-2018"
        :param semester: 学期 数字1或2
        :param week: 周次 数字 1-20
        :return: 二维列表schedule，schedule[i][j]代表周i第j节课的课程。 为了方便，i或j为零0的单元均不使用。
                列表的元素为None，代表没有课程，或描述课程信息的dict，dict例如
                {
                    'classroom': '教4－202', 
                    'name': '技术经济学', 
                    'teacher': '储成祥'
                }
        """
        schedule = [[None for col in range(13)] for row in range(8)]
        if year and semester:
            pass
        else:
            r = self.get(url=URL.zhengfang_class_schedule(self.account))
            soup = BeautifulSoup(r.text.replace('<br>', '\n'), 'lxml')
            trs = soup.select("#Table1 > tr")
            for index, tr in enumerate(trs):
                tds = tr.select('td')
                for td in tds:
                    if len(td.text) > 4:  # 筛选出包含课程信息的单元格
                        for courser in td.text.split('\n\n'):
                            info = courser.split()
                            start_week, end_week = map(int, week_re.search(info[1]).groups())
                            courser_indexs = map(int, courser_indexs_re.search(info[1]).groups()[0].split(','))
                            is_odd_week_only = "单周" in info[1]
                            is_even_week_only = "双周" in info[1]
                            week_day = chinese_rome[info[1][1]]
                            courser = {
                                'name': info[0],
                                'teacher': info[2],
                                'classroom': info[3],
                            }
                            if start_week <= week <= end_week:
                                if (week % 2 == 0 and is_odd_week_only) or (week % 2 == 1 and is_even_week_only):
                                    pass
                                else:
                                    for courser_index in courser_indexs:
                                        schedule[week_day][courser_index] = courser
        return schedule

    @zhengfang_logined
    def get_score(self):
        """
        获取课程成绩和绩点等信息
        :return: dict 
                {'gpa': 4.99,  # GPA
                    'coursers': [{
                        'year': '2015-2016',  # 修读学年
                        'semester': '1',  # 修读学期
                        'code': '00wk00003',  # 课程编号
                        'name': '从"愚昧"到"科学"-科学技术简史',  # 课程中文名
                        'attribute': '任选',  # 课程性质
                        'belong': '全校任选课',  # 课程归属
                        'credit': '2.0',  # 学分
                        'point': '',  # 绩点
                        'score': '81',  # 成绩
                        'minorMark': '0',  # 重修标记
                        'make_up_score': '',  # 补考成绩
                        'retake_score': '',  # 重修成绩
                        'college': '网络课程',  # 开课学院
                        'note': '',  # 备注
                        'retake_mark': '0', # 重修标记
                        'english_name': ''  # 课程英文名
                        }, 
                    ]
                }
        """
        viewstate = self._get_viewstate(url=URL.zhengfang_score(self.account))
        data = {
            'ddlXN': '',
            'ddlXQ': '',
            '__VIEWSTATE': viewstate,
            'Button2': '%D4%DA%D0%A3%D1%A7%CF%B0%B3%C9%BC%A8%B2%E9%D1%AF'
        }
        soup = self._url2soup(method='post', url=URL.zhengfang_score(self.account), data=data)
        result = {'gpa': float(soup.select_one('#pjxfjd > b').text[7:])}
        cols = ['year', 'semester', 'code', 'name', 'attribute', 'belong', 'credit', 'point', 'score', 'minor_mark',
                'make_up_score', 'retake_score', 'college', 'note', 'retake_mark', 'english_name']
        coursers = []
        for tr in soup.select('#Datagrid1  > tr')[1:]:
            courser = {}
            for col, td in zip(cols, tr.select('td')):
                value = td.text.strip()
                courser[col] = value
            coursers.append(courser)
        result['coursers'] = coursers
        return result

    def login(self, account, password):
        """
        登录教务系统 jwxt.njupt.edu.cn
        :param account: 南邮学号
        :param password: 密码
        :return: {'code': 1, "msg": "登录失败"} 或 {'code': 0, 'msg': '登录成功'}
        """
        captcha_code = ZhengfangCaptcha(self._url2image(URL.zhengfang_captcha())).crack()
        data = {
            "__VIEWSTATE": self._get_viewstate(URL.zhengfang_login()),
            'txtUserName': account,
            'TextBox2': password,
            'RadioButtonList1': "%D1%A7%C9%FA",
            "txtSecretCode": captcha_code,
            "Button1": "",
            "hidPdrs": "",
            "hidsc": ""
        }
        result = self._login_execute(url=URL.zhengfang_login(), data=data)
        if result['code'] == 2:
            # 如果验证码错误，尝试递归重复登录
            return self.login(account, password)
        result['success'] = not result['code']
        if result['success']:
            self.verify = True
        return result

    def _login_execute(self, url=None, data=None):
        r = self.post(url=url, data=data)
        if r.ok:
            if "请到信息维护中完善个人联系方式" in r.text:
                self.account = data['txtUserName']
                # self.cookies.save(ignore_discard=True)  # 保存登录信息cookies
                self.verify = True  # 登陆成功, 修改状态  (后期还可能继续修改)
                # self.cookies.load(filename=settings.COOKIES_FILE, ignore_discard=True)
                return {'code': 0, 'msg': '登录成功'}
            elif "密码错误！！" in r.text:
                return {'code': 1, 'msg': '密码错误！！'}
            elif "验证码不正确！！" in r.text:
                return {'code': 2, 'msg': '验证码不正确！！！'}
            else:
                return {'code': 3, 'msg': '未知错误'}
        else:
            return {'code': 1, "msg": "登录失败"}


if __name__ == "__main__":
    pass



# encoding: utf-8
__author__ = 'gaoliang'


class NjuptError(Exception):
    """Base class for exceptions in this module."""

    def __init__(self, *args, **kwargs):
        super(NjuptError, self).__init__(*args, **kwargs)


class ZhengfangNotLogin(NjuptError):
    def __init__(self, message="正方账户未登录"):
        super(ZhengfangNotLogin, self).__init__(message)


class CardNotLogin(NjuptError):
    def __init__(self, message="一卡通账号未登录"):
        super(CardNotLogin, self).__init__(message)



# encoding: utf-8

"""

NJUPT-API的抽象基类，任何对象都可以继承

"""
from io import BytesIO

import requests
import urllib3
from PIL import Image
from bs4 import BeautifulSoup

from njupt import settings

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class Model(requests.Session):
    def __init__(self):
        super(Model, self).__init__()
        # self.cookies = cookiejar.LWPCookieJar(filename=settings.COOKIES_FILE)
        # try:
        #     self.cookies.load(ignore_discard=True)
        # except FileNotFoundError:
        #     pass
        self.verify = False
        self.headers = settings.HEADERS

    def _get_viewstate(self, url=None):
        self.headers['Referer'] = url
        soup = self._url2soup(url=url)
        viewstate = soup.find('input', attrs={"name": "__VIEWSTATE"}).get("value")
        return viewstate

    def _get_viewstategenerator(self, url=None):
        response = self.get(url or url.aolan_login())
        soup = BeautifulSoup(response.content, "lxml")
        viewstate = soup.find('input', attrs={"name": "__VIEWSTATEGENERATOR"}).get("value")
        return viewstate

    def _url2image(self, url=None, method="get", params=None, json=None, data=None, **kwargs):
        """
        图片接口通用请求方法
        :param method: 请求方法，默认为get
        :param url:     请求URL
        :param params:  请求参数
        :param data:    请求数据
        :param data_type:    提交的数据格式(可能是表单类型,也可能是json格式的字符串)
        :param kwargs:  requests支持的参数，比如可以设置代理参数
        :return: pillow的Image对象
        """
        try:  # 出现网络连接问题,直接在该处抛出错误
            r = getattr(self, method)(url, json=json, data=data, params=params, **kwargs)
            return Image.open(BytesIO(r.content))
        except ConnectionError:
            raise ConnectionError("请检查网络连接")

    def _url2soup(self, url=None, method="get", params=None, json=None, data=None, **kwargs):
        """
        html页面通用请求方法
        :param method: 请求方法，默认为get
        :param url:     请求URL
        :param params:  请求参数
        :param data:    请求数据
        :param data_type:    提交的数据格式(可能是表单类型,也可能是json格式的字符串)
        :param kwargs:  requests支持的参数，比如可以设置代理参数
        :return: BeautifulSoup对象
        """
        try:  # 出现网络连接问题,直接在该处抛出错误
            r = getattr(self, method)(url, json=json, data=data, params=params, **kwargs)
        except ConnectionError:
            raise ConnectionError("请检查网络连接")
        else:
            if r.ok:  # 状态码小于400为True
                soup = BeautifulSoup(r.text, 'lxml')
                return soup
            else:  # 处理其他状态码
                raise Exception('请确保能够正常访问当前页面: {}'.format(url))

    def _url2json(self, url=None, method="get", params=None, json=None, data=None, **kwargs):
        """
        json接口通用请求方法
        :param method: 请求方法, 默认为get
        :param url:     请求URL
        :param params:  请求参数
        :param data:    请求数据
        :param data_type:    提交的数据格式(可能是表单类型,也可能是json格式的字符串)
        :param kwargs:  requests支持的参数，比如可以设置代理参数
        :return: json转换出的字典
        """
        try:  # 出现网络连接问题,直接在该处抛出错误
            r = getattr(self, method)(url, json=json, data=data, params=params, **kwargs)
            return r.json()
        except ConnectionError:
            raise ConnectionError("请检查网络连接")


if __name__ == "__main__":
    test_model = Model()



"""
方便收集正方验证码的简单打码工具
"""
import tkinter
from io import BytesIO

import requests
from PIL import Image, ImageTk
from bs4 import BeautifulSoup


class CaptchaGUI:
    captcha_url = "http://jwxt.njupt.edu.cn/CheckCode.aspx"
    login_url = "http://jwxt.njupt.edu.cn/default2.aspx"

    def __init__(self):

        self.s = requests.session()
        self.get_viewstate()
        self.r = self.s.get(self.captcha_url)
        self.im = Image.open(BytesIO(self.r.content))

        self.root = tkinter.Tk()
        self.tkimg = ImageTk.PhotoImage(self.im)
        self.imgLabel = tkinter.Label(self.root, image=self.tkimg)
        self.imgLabel.pack()
        self.message = tkinter.Entry(self.root)
        self.message.pack()
        self.root.bind('<Return>', self.judge_and_save)
        self.root.mainloop()

    def get_viewstate(self):
        r = self.s.get(self.login_url)
        soup = BeautifulSoup(r.content, "lxml")
        self.viewstate = soup.find('input', attrs={"name": "__VIEWSTATE"}).get("value")

    def judge_and_save(self, event):
        captcha_value = self.message.get()
        print(captcha_value)
        data = {
            "__VIEWSTATE": self.viewstate,
            'txtUserName': "",  # 账号
            'TextBox2': "",  # 密码
            'RadioButtonList1': "%D1%A7%C9%FA",
            "Button1": "",
            "txtSecretCode": captcha_value,
            "hidPdrs": "",
            "hidsc": ""
        }

        r = self.s.post(self.login_url, data=data)
        if "请到信息维护中完善个人联系方式" in r.text:
            print("成功！")
            with open("captchas/{}.gif".format(captcha_value), 'wb+') as f:
                f.write(self.r.content)
        else:
            print("验证码输错了")
        self.get_viewstate()
        self.r = self.s.get(self.captcha_url)
        self.im = Image.open(BytesIO(self.r.content))
        self.tkimg = ImageTk.PhotoImage(self.im)
        self.imgLabel.config(image=self.tkimg)
        self.message.delete(0, 'end')


if __name__ == "__main__":
    captcha_gui = CaptchaGUI()



import os

import math
import pickle

from PIL import Image

BLACK = 0
WHITE = 255

current_dir = os.path.dirname(os.path.abspath(__file__))


class VectorTools:
    # 计算矢量大小
    def magnitude(self, concordance):
        total = 0
        for word, count in concordance.items():
            total += count ** 2
        return math.sqrt(total)

    # 计算矢量之间的 cos 值
    def relation(self, concordance1, concordance2):
        relevance = 0
        topvalue = 0
        for word, count in concordance1.items():
            if word in concordance2:
                topvalue += count * concordance2[word]
        return topvalue / (self.magnitude(concordance1) * self.magnitude(concordance2))


class CardCaptcha:
    """
    一卡通系统的验证码工具类
    """

    def __init__(self, im):
        """
        :param im: PIL.image对象
        """
        self.im = im.convert('L')
        self._reduce_noise()

    def _reduce_noise(self):

        threshold = 200
        # 去除杂色点
        for x in range(self.im.width):
            for y in range(self.im.height):
                pix = self.im.getpixel((x, y))
                if pix > threshold:
                    self.im.putpixel((x, y), WHITE)
                else:
                    self.im.putpixel((x, y), BLACK)

    def handle_split_image(self):
        # 切割验证码，返回包含五个字符图像的列表
        y_min, y_max = 0, 82
        split_lines = [7, 49, 91, 133, 175, 217]
        ims = [self.rotate_img(self.im.crop([u, y_min, v, y_max])) for u, v in zip(split_lines[:-1], split_lines[1:])]
        return ims

    def rotate_img(self, im):
        """
        根据图像在x轴方向投影大小确定字符的摆放方向
        :param im: PIL.Image object
        :return: rotated Image object
        """
        min_count = 1000
        final_angle = 0
        for angle in range(-45, 45):
            x_count = 0
            ti = im.rotate(angle, expand=True)
            for x in range(ti.width):
                for y in range(ti.height):
                    if ti.getpixel((x, y)) == WHITE:
                        x_count += 1
                        break
            if x_count < min_count:
                min_count = x_count
                final_angle = angle
        im = im.rotate(final_angle, expand=False)
        return im

    @staticmethod
    def buildvector(im):
        d1 = {}
        count = 0
        for i in im.getdata():
            d1[count] = 1 if i > 0 else 0
            count += 1
        return d1

    def _abs_image(self):
        self.im = self.im.cro(self.im.getbbox())

    def crack(self):
        result = []
        v = VectorTools()
        # 装载训练数据集
        with open(os.path.join(current_dir, 'imageset.dat'), 'rb+') as f:
            imageset = pickle.load(f)
        for letter in self.handle_split_image():
            guess = []
            for image in imageset:
                for x, y in image.items():
                    if len(y) != 0:
                        guess.append((v.relation(y[0], self.buildvector(letter)), x))
            guess.sort(reverse=True)
            neighbors = guess[:10]  # 距离最近的十个向量
            class_votes = {}  # 投票
            for neighbor in neighbors:
                class_votes.setdefault(neighbor[-1], 0)
                class_votes[neighbor[-1]] += 1
            sorted_votes = sorted(class_votes.items(), key=lambda x: x[1], reverse=True)
            result.append(sorted_votes[0][0])
        return ''.join(result)

    def __str__(self):
        return self.crack()


if __name__ == "__main__":
    im = Image.open("captchas/12272.gif")
    im.show()
    print(CardCaptcha(im).crack())



import os

import numpy as np
from sklearn.externals import joblib

BLACK = 0
WHITE = 255

current_dir = os.path.dirname(os.path.abspath(__file__))


class ZhengfangCaptcha:
    """
    svm 识别正方验证码
    """
    def __init__(self, im):
        """
        :param im: PIL.image对象
        """
        self.im = im
        self._reduce_noise()

    def _reduce_noise(self):

        # 去除杂色点
        for x in range(self.im.width):
            for y in range(self.im.height):
                pix = self.im.getpixel((x, y))
                # if pix in [43, 85, 128, 170]:
                if pix == 43:
                    # if pix in [43, 85, 86, 127, 128, ]:
                    self.im.putpixel((x, y), WHITE)
                else:
                    self.im.putpixel((x, y), BLACK)

        # 去除单像素噪点并进行二值化(八值法)
        for x in range(self.im.width):
            for y in range(self.im.height):
                count = 0
                if x != 0 and y != 0 and x != self.im.width - 1 and y != self.im.height - 1:
                    for i in range(-1, 2):
                        for j in range(-1, 2):
                            tx = x + i
                            ty = y + j
                            if self.im.getpixel((tx, ty)) == BLACK:
                                count += 1
                if self.im.getpixel((x, y)) == WHITE and count == 8:
                    self.im.putpixel((x, y), BLACK)

    def handle_split_image(self):
        # 切割验证码，返回包含四个字符图像的列表
        y_min, y_max = 0, 22
        split_lines = [5, 17, 29, 41, 53]
        ims = [self.rotate_img(self.im.crop([u, y_min, v, y_max])) for u, v in zip(split_lines[:-1], split_lines[1:])]
        return ims

    @staticmethod
    def getletter(im):
        # 获取特征值
        all = []
        for x in range(im.width):
            all_t = []
            for y in range(im.height):
                all_t.append(1 if im.getpixel((x, y)) > 0 else 0)
            all += all_t
        return all

    def rotate_img(self, im):
        """
        根据图像在x轴方向投影大小确定字符的摆放方向
        :param im: PIL.Image object
        :return: rotated Image object
        """
        min_count = 1000
        final_angle = 0
        for angle in range(-45, 45):
            x_count = 0
            ti = im.rotate(angle, expand=True)
            for x in range(ti.width):
                for y in range(ti.height):
                    if ti.getpixel((x, y)) == WHITE:
                        x_count += 1
                        break
            if x_count < min_count:
                min_count = x_count
                final_angle = angle
        im = im.rotate(final_angle, expand=False)
        return im

    def _abs_image(self):
        self.im = self.im.cro(self.im.getbbox())

    def crack(self):
        clf = joblib.load(os.path.join(current_dir, 'letter.pkl'))
        temp = []
        for i, img in enumerate(self.handle_split_image()):
            data = self.getletter(img)
            data = np.array([data])
            oneLetter = clf.predict(data)[0]
            temp.append(oneLetter)
        return "".join(temp)

    def __str__(self):
        return self.crack()



# encoding: utf-8

# UserAgent list
user_agent_list = [
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_3) "
    "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/59.0.3071.115 Safari/537.36"
]

USER_AGENT = user_agent_list[0]

HEADERS = {
    'User-Agent': USER_AGENT
}
#
# # 教务系统headers
# JWXT_HEADERS = {
#     "Host": "jwxt.njupt.edu.cn",
#     "Referer": "http://jwc.njupt.edu.cn/",
#     'User-Agent': USER_AGENT
# }

# # COOKIE 存储位置
# COOKIES_FILE = os.path.join(os.path.expanduser('~'), "cookies.txt")
#
# # BASE_DIR, 使用os.path.join(BASE_DIR, ...)
# BASE_DIR = os.path.dirname(os.path.abspath(__file__))



import hashlib
import os
import shutil

from PIL import Image
from sklearn.externals import joblib
from sklearn.svm import SVC

BLACK = 0
WHITE = 255
import time

from .zhengfang_captcha import ZhengfangCaptcha


def spilt2chars():
    """
    分割已有的数据为字符并保存
    """
    try:
        shutil.rmtree('captcha_chars')
    except:
        pass
    os.mkdir("captcha_chars")
    values = "abcdefghijklmnopqrstuvwxyz1234567890"
    for value in values:
        os.mkdir('captcha_chars/{}'.format(value))

    file_names = os.listdir('captchas')
    for file_name in file_names:  #
        if not os.path.isdir(file_name) and file_name != '.DS_Store':
            values = file_name[:4]
            im = Image.open('captchas/{}'.format(file_name))
            captcha = ZhengfangCaptcha(im)
            for im_part, value in zip(captcha.handle_split_image(), values):
                m = hashlib.md5()
                m.update("{}{}".format(time.time(), value).encode('utf8'))
                im_part.save("captcha_chars/{}/{}.png".format(value, m.hexdigest()))


def buildvector(im):
    d1 = {}
    count = 0
    for i in im.getdata():
        d1[count] = i
        count += 1
    return d1


# 提取字母的svm特征值
def getletter(im):
    all = []
    for x in range(im.width):
        all_t = []
        for y in range(im.height):
            all_t.append(1 if im.getpixel((x, y)) > 0 else 0)
        all += all_t
    return all


# 提取特征值
def extractLetters(path):
    """
    :param path: 存放已经分割好的字符文件夹的父目录路径
    :return: 提取的特征值
    """
    x = []
    y = []
    # 遍历文件夹 获取下面的目录
    for root, sub_dirs, files in os.walk(path):
        for dir in sub_dirs:
            # 获得每个文件夹的图片
            for fileName in os.listdir(path + '/' + dir):
                print(dir, fileName)
                if fileName != "Thumbs.db" and fileName != ".DS_Store":
                    # 打开图片
                    x.append(getletter(Image.open(path + '/' + dir + '/' + fileName)))
                    y.append(dir)
    print(x)
    return x, y


# svm训练
def trainSVM():
    array = extractLetters('captcha_chars')
    # 使用向量机SVM进行机器学习
    letterSVM = SVC(kernel="linear", C=1, probability=True).fit(array[0], array[1])
    # 生成训练结果
    joblib.dump(letterSVM, 'letter.pkl')


if __name__ == "__main__":
    spilt2chars()
    trainSVM()



"""
奥兰系统验证码识别，利用向量空间余弦相似度实现
"""
import os
import pickle
import requests
import math
from njupt.urls import URL
from PIL import Image
from io import BytesIO

# 对图像进行纵向分割的分割点

letters = [(3, 11), (23, 31), (43, 51), (63, 71)]
current_dir = os.path.dirname(os.path.abspath(__file__))


class VectorCompare:
    # 计算矢量大小
    def magnitude(self, concordance):
        total = 0
        for word, count in concordance.items():
            total += count ** 2
        return math.sqrt(total)

    # 计算矢量之间的 cos 值
    def relation(self, concordance1, concordance2):
        relevance = 0
        topvalue = 0
        for word, count in concordance1.items():
            if word in concordance2:
                topvalue += count * concordance2[word]
        return topvalue / (self.magnitude(concordance1) * self.magnitude(concordance2))


# 将图片转换为矢量
def buildvector(im):
    d1 = {}
    count = 0
    for i in im.getdata():
        d1[count] = i
        count += 1
    return d1


class AolanCaptcha:
    # 分割点
    letters = [(3, 11), (23, 31), (43, 51), (63, 71)]

    def __init__(self, im):
        # 路径或BytesIO或文件时
        if isinstance(im, str) or isinstance(im, BytesIO) or type(im) == "file":
            self.im = Image.open(im)
        elif isinstance(im, Image.Image):
            self.im = im
        else:
            raise TypeError("错误的类型")
        self.binarize()

    def binarize(self):

        # 灰度处理
        self.im = self.im.convert('L')

        # 二值处理
        threshold = 220
        table = []
        for i in range(256):
            if i < threshold:
                table.append(0)
            else:
                table.append(1)
        self.im = self.im.point(table, '1')

    def crack(self):
        v = VectorCompare()
        # 装载训练数据集
        with open(os.path.join(current_dir, 'imageset.dat'), 'rb+') as f:
            imageset = pickle.load(f)
        # 对验证码图片进行切割
        result = []
        for letter in letters:
            final = self.im.crop((letter[0], 0, letter[1], self.im.size[1]))
            guess = []
            # 将切割得到的验证码小片段与每个训练片段进行比较
            for image in imageset:
                for x, y in image.items():
                    if len(y) != 0:
                        guess.append((v.relation(y[0], buildvector(final)), x))
            guess.sort(reverse=True)
            result.append(guess[0][1])
        return "".join(result)

    def __str__(self):
        return self.crack()


if __name__ == "__main__":
    r = requests.get(URL.aolan_captcha())
    im = Image.open(BytesIO(r.content))
    print(AolanCaptcha(im))



import hashlib

from njupt import settings
from njupt.models.base import Model
from njupt.urls import URL
from njupt.utils import AolanCaptcha


class Aolan(Model):
    def login(self, account, password):
        """
        登录奥兰系统 jwxt.njupt.edu.cn
        :param account: 南邮学号、考生号、身份证号
        :param password: 密码
        :return: {'r': 1, "msg": "登录失败"} 或 {'r': 0, 'msg': '登录成功'}
        """
        captcha_code = AolanCaptcha(self._url2image(URL.aolan_captcha())).crack()
        data = {
            "__VIEWSTATE": self._get_viewstate(URL.aolan_login()),
            '__VIEWSTATEGENERATOR': captcha_code,
            'userbh': account,
            'pas2s': hashlib.md5(password.upper().encode('utf8')).hexdigest(),
            "vcode": (URL.aolan_captcha()),
            "cw": "",
            "xzbz": "1",
        }
        return self._login_execute(url=URL.aolan_login(), data=data)

    def _login_execute(self, url=None, data=None):
        r = self.post(url=url, data=data)
        if r.ok:
            if "辅导员评议" in r.text:
                self.cookies.save(ignore_discard=True)  # 保存登录信息cookies
                self.cookies.load(filename=settings.COOKIES_FILE, ignore_discard=True)
                return {'r': 0, 'msg': '登录成功'}
            else:
                return {'r': 1, 'msg': '检查账号密码验证码是否正确'}
        else:
            return {'r': 1, "msg": "登录失败"}



import hashlib
import os
import pickle
import shutil
from PIL import Image

from njupt.utils import ZhengfangCaptcha

BLACK = 0
WHITE = 255
import time


def spilt2chars():
    """
    分割已有的数据为字符并保存
    """
    try:
        shutil.rmtree('captcha_chars')
    except:
        pass
    os.mkdir("captcha_chars")
    values = "abcdefghijklmnopqrstuvwxyz1234567890"
    for value in values:
        os.mkdir('captcha_chars/{}'.format(value))

    file_names = os.listdir('captchas')
    for file_name in file_names:  #
        if not os.path.isdir(file_name) and file_name != '.DS_Store':
            values = file_name[:4]
            im = Image.open('captchas/{}'.format(file_name))
            captcha = ZhengfangCaptcha(im)
            for im_part, value in zip(captcha.handle_split_image(), values):
                m = hashlib.md5()
                m.update("{}{}".format(time.time(), value).encode('utf8'))
                im_part.save("captcha_chars/{}/{}.png".format(value, m.hexdigest()))


if __name__ == "__main__":
    # spilt2chars()
    iconset = list('qwertyuiopasdfghjklzcxvbnm1234567890')
    # 将图像数据转为向量数据并保存
    imageset = []
    for letter in iconset:
        try:
            for img in os.listdir('captcha_chars/{}/'.format(letter)):
                temp = []
                if img != "Thumbs.db" and img != ".DS_Store":
                    temp.append(ZhengfangCaptcha.buildvector(Image.open("captcha_chars/{}/{}".format(letter, img))))
                    print(temp)

                imageset.append({letter: temp})
        except:
            print(letter)

    with open('imageset.dat', 'wb+') as f:
        pickle.dump(imageset, f)



import os
import pickle

from PIL import Image


# 将图片转换为矢量
def buildvector(im):
    d1 = {}
    count = 0
    for i in im.getdata():
        d1[count] = i
        count += 1
    return d1


if __name__ == "__main__":
    # 字符集
    iconset = ['2', '3', '4', '5', '6', '7', '8', '9']
    # 将图像数据转为向量数据并保存
    imageset = []
    for letter in iconset:
        for img in os.listdir('captcha_chars/{}/'.format(letter)):
            temp = []
            if img != "Thumbs.db" and img != ".DS_Store":
                temp.append(buildvector(Image.open("captcha_chars/{}/{}".format(letter, img))))
            imageset.append({letter: temp})

    with open('imageset.dat', 'wb+') as f:
        pickle.dump(imageset, f)



# encoding: utf-8
import base64
import datetime
import json

from njupt.decorators.card_logined import card_logined
from njupt.error import NjuptError
from njupt.models.base import Model
from njupt.urls import URL
from njupt.utils import CardCaptcha

AIDS = {
    'elec_xianlin': '0030000000005101',
    'elec_sanpailou': '0030000000005102',
    'net': '0030000000000301'
}


class Card(Model):
    def __init__(self, account=None, password=None):
        super(Card, self).__init__()
        self.aid = None

        # 该系统使用aid区分业务类型

        self.inner_account = None
        if account and password:
            self.account = account
            self.login(account, password)

    def login(self, account, password):
        """
        登录校园卡网站
        :param str account: 校园卡卡号
        :param str password: 校园卡查询密码
        :return: {'code': 1, "msg": "登录失败，请重试"} 或 {'code': 0, 'msg': '登录成功'}
        """
        captcha_code = CardCaptcha(self._url2image(URL.card_captcha())).crack()
        data = {
            "sno": account,
            "pwd": base64.b64encode(str(password).encode("utf8")),
            "ValiCode": captcha_code,
            "remember": 1,
            "uclass": 1,
            "zqcode": "",
            "json": True,
        }

        result = self._login_execute(url=URL.card_login(), data=data)
        if result['code'] == 2:
            # 如果验证码错误，尝试递归重复登录
            return self.login(account, password)
        result['success'] = not result['code']
        if result['success']:
            self.verify = True
        return result

    def _login_execute(self, url=None, data=None):
        result = self._url2json(url=url, data=data, method="post")
        if result['IsSucceed']:
            return {'code': 0, 'msg': '登录成功'}
        else:
            if result['Msg'] == "验证码错误":
                return {'code': 2, 'msg': '验证码错误'}
            else:
                return {'code': 1, 'msg': result['Msg']}

    def _get_inner_account(self):
        """
        获取对应的内部账号，部分接口参数需要
        :return: 校园卡号对应的系统内部账号 
        """
        if not self.inner_account:
            info = self._url2json(URL.card_info(), method='post', data={'json': True})
            result = json.loads(info['Msg'])
            self.inner_account = result['query_card']['card'][0]['account']
        return self.inner_account

    @card_logined
    def get_balance(self):
        """
        查询余额
        :return: dict
                {
                    'balance': 10.01,  # 到账余额
                    'unsettle_balance': 0.01  # 过渡余额
                    'total': 10.02  # 总余额
                }
        """
        info = self._url2json(URL.card_info(), method='post', data={'json': True})
        result = json.loads(info['Msg'])
        balance = int(result['query_card']['card'][0]['db_balance']) / 100  # 到账余额
        unsettle_balance = int(result['query_card']['card'][0]['unsettle_amount']) / 100  # 过渡余额
        return {
            'balance': balance,  # 到账余额
            'unsettle_balance': unsettle_balance,  # 过渡余额
            'total': balance + unsettle_balance,  # 总余额
        }

    @card_logined
    def get_net_balance(self):
        """
        获取Dr.com的上网费用余额
        :return: float  2.33
        """
        data = {
            'jsondata': '{"query_net_info": {"aid": "%s", "account": "%s", "payacc": ""}}'  # 无法使用format格式化
                        % (AIDS['net'], self._get_inner_account()),
            'funname': 'synjones.onecard.query.net.info',
            'json': True,
        }
        result = self._url2json(url=URL.card_common(), data=data, method='post')
        result = json.loads(result['Msg'])
        return float(result['query_net_info']['errmsg'][12:][:-1])

    @card_logined
    def recharge(self, amount=0.01):
        """
        从绑定的银行卡中扣款充值余额
        :param amount: 充值金额，默认为0.01元
        :return: dict
            {   
                'success':True  # 转账是否成功
                'code': 0,  # 状态码
                'msg': '转账成功'  # 附加信息
            }
        """
        data = {
            'account': self._get_inner_account(),
            'acctype': '###',
            'tranamt': int(100 * amount),
            'qpwd': '',
            'paymethod': '2',
            'paytype': '使用绑定的默认账号',
            'client_type': 'web',
            'json': True,
        }
        info = self._url2json(URL.card_recharge(), 'post', data)
        info = json.loads(info['Msg'])['transfer']
        result = {
            'code': int(info['retcode']),
            'msg': info['errmsg'],
            'success': not bool(int(info['retcode']))
        }
        return result

    @card_logined
    def recharge_net(self, amount=0.01):
        """
        充值网费（从校园卡余额中）
        :param amount: 金额 如 2.33
        :return: dict
            {
                'success':True, 
                'code' : 0,
                'Msg' : '充值成功'
            }
        """
        data = {
            'paytype': 1,
            'aid': AIDS['net'],
            'account': self._get_inner_account(),
            'tran': int(amount * 100),
            'netacc': '{"netacc": "", "bal": null, "pkgid": null, "lostflag": null, "freezeflag": null, '
                      '"expflag": null,"statustime": null, "duration": null, "starttime": null, "pkgtab": []}',
            'pkgflag': 'none',
            'pkg': '{}',
            'acctype': '###',
            'qpwd': '',
            'json': True
        }
        result = self._url2json(url=URL.card_net_pay(), method='post', data=data)
        result = json.loads(result['Msg'])
        return {
            'success': not int(result['pay_net_gdc']['retcode']),
            'code': int(result['pay_net_gdc']['retcode']),
            'msg': result['pay_net_gdc']['errmsg']
        }

    def _get_build_ids(self, aid):
        """获取建筑的id"""
        data = {
            'jsondata': '{"query_elec_building": {"aid": "%s", "account": "%s", "area": {"area": "", "areaname": ""}}}'
                        % (aid, self._get_inner_account()),
            'funname': 'synjones.onecard.query.elec.building',
            'json': True,
        }
        result = self._url2json(data=data, url=URL.card_common(), method='post')
        result = json.loads(result['Msg'])
        # - -!
        building_id = {}
        for build in result['query_elec_building']['buildingtab']:
            building_id[build['building']] = build['buildingid']
        return building_id

    @card_logined
    def recharge_xianlin_elec(self, amount, building_name, room_id):
        """
        充值仙林校区的寝室电费
        :param amount: 金额 0.01 2.33
        :param building_name: 楼栋名称，例如 "兰苑11栋" 
        :param room_id: 房间号，例如4030 -> 403公共 4031 -> 403 1寝空调 4032 -> 403 2寝空调
        :return: dict {'code': 0, 'msg': '缴费成功！', 'success': True}
        """
        try:
            buiding_id = self._get_build_ids(AIDS['elec_xianlin'])[building_name]
            return self._recharge_electricity(amount, elec_aid=AIDS['elec_xianlin'], building_id=buiding_id,
                                              building=building_name, room_id=room_id)
        except KeyError:
            raise NjuptError("不存在的楼栋")

    @card_logined
    def recharge_sanpailou_elec(self, amount, building_name, room_id):
        """
        充值三牌楼校区的寝室电费，参数参考仙林校区（未测试
        """
        try:
            buiding_id = self._get_build_ids(AIDS['elec_sanpailou'])[building_name]
            return self._recharge_electricity(amount, elec_aid=AIDS['elec_sanpailou'], building_id=buiding_id,
                                              building=building_name, room_id=room_id)
        except KeyError:
            raise NjuptError("不存在的楼栋")

    def _recharge_electricity(self, amount, elec_aid, building_id, building, room_id):
        data = {
            "acctype": "###",
            "paytype": 1,
            "aid": elec_aid,
            "account": self._get_inner_account(),
            "tran": int(amount * 100),
            "roomid": room_id,
            "room": "",
            "floorid": "",
            "floor": "",
            "buildingid": building_id,
            "building": building,
            "areaid": "",
            "areaname": "",
            "json": True
        }

        r = json.loads(self._url2json(URL.card_elec_pay(), 'post', data=data)['Msg'])
        return {
            'success': not bool(int(r['pay_elec_gdc']['retcode'])),
            'code': int(r['pay_elec_gdc']['retcode']),
            'msg': r['pay_elec_gdc']['errmsg'],
        }

    @card_logined
    def get_bill(self, start_date=(datetime.datetime.now() - datetime.timedelta(days=30)).strftime("%Y-%m-%d"),
                 end_date=datetime.datetime.now().strftime("%Y-%m-%d"), rows=100, page=1):
        """
        查询校园卡消费记录，默认为最近一个月的消费记录
        :param start_date: 查询区间的开始时间 "2017-12-27"
        :param end_date:  查询区间的结束时间 "2018-01-26"
        :param rows: 一次查询返回的最大记录数量,默认为100条记录
        :param page: 如果结果数量有多页，决定返回第几页。
        :return: list 
            {'recodes': 
                [{'balance': 39.71,  # 余额
                  'change': -5, # 变动
                  'comment': '未知系统,交电费', # 注释
                  'department': '仙林售电处', # 操作部门
                  'time': '2018-01-26 20:55:40', # 时间
                  'type': '代扣代缴', # 类型
                  'week': '星期五'}, # 星期
                 {'balances': 39.71,
                  'change': -7.5,
                  'comment': '',
                  'department': '一餐厅二楼清真食堂',
                  'time': '2018-01-24 17:09:36',
                  'type': '持卡人消费',
                  'week': '星期三'}
                   ... ],
            'total': 52 # 总的记录数
            'total_pages':2 # 总页数
            'page':1  # 当前的页码
            }
        """
        data = {
            "sdate": start_date,
            "edate": end_date,
            "account": self._get_inner_account(),
            "page": page,
            "rows": rows
        }
        temp = self._url2json(url=URL.card_bill(), method="post", data=data)
        result = {'total': temp['total'], 'recodes': [], 'total_pages': temp['total'] // rows + 1, 'page': page}
        if temp['rows']:
            for row in temp['rows']:
                result['recodes'].append({
                    'balance': row['CARDBAL'],
                    'department': row['MERCNAME'].strip(),
                    'comment': row['JDESC'].strip(),
                    'change': row['TRANAMT'],
                    'week': row['XQ'],
                    'time': row['OCCTIME'],
                    'type': row['TRANNAME'].strip(),
                })
        return result


if __name__ == "__main__":
    card = Card('110201500467700', '117991')
    print(card.recharge_xianlin_elec(3, '兰苑11栋', 4031))



# encoding: utf-8

"""
用户认证装饰器(判断用户是否已经登录)
"""

import requests
import requests.utils

from njupt.error import ZhengfangNotLogin
from njupt.urls import URL


def zhengfang_logined(func):
    def wrapper(self, *args, **kwargs):
        if self.verify:  # 利用状态量进行状态的判定
            return func(self, *args, **kwargs)
        else:
            # 先判断有没有cookie文件, 再判断cookie是否有效
            if 'WEB' in requests.utils.dict_from_cookiejar(self.cookies):
                # 判断教务系统是否登录成功，根据正方的一个地址无需参数的地址
                r = self._url2soup(method="get", url=URL.zhengfang_logintest())
                if 'Object moved to ' not in r.text:
                    self.verify = True
                    return func(self, *args, **kwargs)
            raise ZhengfangNotLogin

    return wrapper



# encoding: utf-8
"""
通用ＵＲＬ类,NJUPT各个网站的ＵＲＬ接口地址
"""


class URL(object):
    HOST = "https://www.njupt.edu.cn"
    ZHENGFANG_HOST = "http://jwxt.njupt.edu.cn"
    AOLAN_HOST = "http://stu.njupt.edu.cn"
    LIB_HOST = "http://202.119.228.6:8080"
    CARD_HOST = "http://yktapp.njupt.edu.cn:8070"

    @classmethod
    def zhengfang_logintest(cls):
        return cls.ZHENGFANG_HOST + '/content.aspx'

    @classmethod
    def zhengfang_score(cls, account):
        return cls.ZHENGFANG_HOST + '/xscj_gc.aspx?xh={}&xm=%B8%DF%C1%C1&gnmkdm=N121605'.format(account)

    @classmethod
    def zhengfang_class_schedule(cls, account):
        return cls.ZHENGFANG_HOST + '/xskbcx.aspx?xh={}&xm=%B8%DF%C1%C1&gnmkdm=N121603'.format(account)

    @classmethod
    def zhengfang_grade(cls, account):
        return cls.ZHENGFANG_HOST + '/xsdjkscx.aspx?xh={}&xm=%B8%DF%C1%C1&gnmkdm=N121606'.format(account)

    @classmethod
    def zhengfang_captcha(cls):
        # 教务系统验证码
        return cls.ZHENGFANG_HOST + '/CheckCode.aspx'

    @classmethod
    def zhengfang_login(cls):
        # 教务系统登录
        return cls.ZHENGFANG_HOST + '/default2.aspx'

    @classmethod
    def aolan_captcha(cls):
        return cls.AOLAN_HOST + '/VCode.aspx'

    @classmethod
    def aolan_login(cls):
        # 奥兰系统登录
        return cls.AOLAN_HOST + "/LOGIN.ASPX"

    @classmethod
    def lib_login(cls):
        # 图书馆登录
        return cls.LIB_HOST + '/reader/redr_verify.php'

    @classmethod
    def lib_captcha(cls):
        # 图书馆验证码
        return cls.LIB_HOST + '/reader/captcha.php'

    @classmethod
    def card_login(cls):
        # 一卡通登录
        return cls.CARD_HOST + '/Login/LoginBySnoQuery'

    @classmethod
    def card_captcha(cls):
        # 一卡通验证码
        return cls.CARD_HOST + '/Login/GetValidateCode'

    @classmethod
    def card_info(cls):
        # 一卡通基础信息
        return cls.CARD_HOST + '/User/GetCardInfoByAccountNoParm'

    @classmethod
    def card_bill(cls):
        # 一卡通详单
        return cls.CARD_HOST + '/Report/GetMyBill'

    @classmethod
    def card_recharge(cls):
        # 一卡通充值
        return cls.CARD_HOST + '/User/Account_Pay'

    @classmethod
    def card_common(cls):
        # 一卡通Dr.com信息
        return cls.CARD_HOST + '/Tsm/TsmCommon'

    @classmethod
    def card_net_pay(cls):
        # 一卡通网费充值
        return cls.CARD_HOST + '/Tsm/Net_Pay'

    @classmethod
    def card_elec_pay(cls):
        return cls.CARD_HOST + '/Tsm/Elec_Pay'



"""
方便收集南邮一卡通网站验证码的简单打码工具
"""
import base64
import tkinter
from io import BytesIO

import requests
from PIL import Image, ImageTk


class CaptchaGUI:
    BASE_URL = "http://yktapp.njupt.edu.cn:8070"
    CAPTCHA_URL = BASE_URL + "/Login/GetValidateCode"
    LOGIN_URL = BASE_URL + "/Login/LoginBySnoQuery"

    def __init__(self):
        self.pwd = ""  # your njupt card number
        self.account = ""  # your njupt card password
        self.s = requests.session()
        self.r = self.s.get(self.CAPTCHA_URL)
        self.im = Image.open(BytesIO(self.r.content))
        self.root = tkinter.Tk()
        self.tkimg = ImageTk.PhotoImage(self.im)
        self.imgLabel = tkinter.Label(self.root, image=self.tkimg)
        self.imgLabel.pack()
        self.message = tkinter.Entry(self.root)
        self.message.pack()
        self.root.bind('<Return>', self.judge_and_save)
        self.root.mainloop()

    def judge_and_save(self, event):
        captcha_value = self.message.get()
        print(captcha_value)

        data = {
            "sno": self.account,
            "pwd": base64.b64encode(self.pwd.encode("utf8")),
            "ValiCode": captcha_value,
            "remember": 1,
            "uclass": 1,
            "zqcode": "",
            "json": True,
        }

        r = self.s.post(self.LOGIN_URL, data=data)
        if r.json()["IsSucceed"]:
            print("成功！")
            with open("captchas/{}.gif".format(captcha_value), 'wb+') as f:
                f.write(self.r.content)
        else:
            print(r.json())
            print("验证码输错了")
        self.r = self.s.get(self.CAPTCHA_URL)
        self.im = Image.open(BytesIO(self.r.content))
        self.tkimg = ImageTk.PhotoImage(self.im)
        self.imgLabel.config(image=self.tkimg)
        self.message.delete(0, 'end')


if __name__ == "__main__":
    captcha_gui = CaptchaGUI()



try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup

with open("README.MD", encoding='utf-8') as f:
    long_description = f.read()

install_requires = open("requirements.txt").readlines()

setup(
    name='njupt',
    version='0.1',
    platforms='any',
    packages=find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    url='https://github.com/gaoliang/NJUPT-API',
    license='MIT License',
    author='gaoliang',
    install_requires=install_requires,
    author_email='gaoliangim@gmail.com',
    package_data={
        # If any package contains *.txt or *.rst files, include them:
        '': ['*.dat'],
    },
    description='njupt api for humans',
    long_description=long_description,
)



# encoding: utf-8

"""
用户认证装饰器(判断一卡通是否已经登录)
"""
from njupt.error import CardNotLogin


def card_logined(func):
    def wrapper(self, *args, **kwargs):
        if self.verify:  # 利用状态量进行状态的判定
            return func(self, *args, **kwargs)
        else:
            raise CardNotLogin

    return wrapper
